package aptTools

import (
	"bytes"
	sdk "github.com/imshuai/alistsdk-go"
	"log"
	"net/http"
	"path/filepath"
	"strings"
)

type ActionType int

const (
	Copy ActionType = iota + 1
	Move
)

type TaskType string

const (
	Upload              TaskType = "upload"
	CopyTask            TaskType = "copy"
	Aria2Download       TaskType = "aria2_down"
	Aria2Transfer       TaskType = "aria2_transfer"
	QbittorrentDownload TaskType = "qbit_down"
	QbittorrentTransfer TaskType = "qbit_transfer"
)

type AlistAPI struct {
	URL      string
	Username string
	Password string
	Headers  map[string]string
	Client   *sdk.Client
	User     *sdk.User
}

type ListDirectoryPayload struct {
	Path     string `json:"path"`
	Password string `json:"password"`
	Page     int    `json:"page"`
	PerPage  int    `json:"per_page"`
	Refresh  bool   `json:"refresh"`
}
type FileItem struct {
	Name             string `json:"name"`
	FolderName       string `json:"folder_name"`
	ParentFolderName string `json:"parent_folder_name"`
	IsDir            bool   `json:"is_dir"`
	Path             string `json:"path"`
	DownloadsPath    string `json:"downloads_path"`
	SrcDir           string `json:"src_dir"`
	DstDir           string `json:"dst_dir"`
	DownloadsUrl     string `json:"downloads_url"`
}

func NewAlistAPI(url, username, password string) *AlistAPI {
	return &AlistAPI{
		URL:      url,
		Username: username,
		Password: password,
		Headers: map[string]string{
			"User-Agent":   "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/87.0.4280.88 Safari/537.36",
			"Content-Type": "application/json",
		},
		Client: sdk.NewClient(url, username, password, false, 60),
	}
}

func (api *AlistAPI) Login() error {

	user, err := api.Client.Login()
	if err != nil {
		return err
	}

	api.User = user

	return nil
}

func (api *AlistAPI) request(method, endpoint string, body []byte) (*http.Response, error) {
	req, err := http.NewRequest(method, api.URL+endpoint, bytes.NewBuffer(body))
	if err != nil {
		return nil, err
	}

	for key, value := range api.Headers {
		req.Header.Set(key, value)
	}

	client := &http.Client{}
	return client.Do(req)
}

func (api *AlistAPI) ListDirectory(path string) error {

	files, err := api.Client.List(path, "", 0, 1000, false)
	if err != nil {
		return err
	}
	for i, file := range files {
		println(i)
		println(file.Name)
	}

	return nil

}

// isMediaFile checks if the file has a media file extension
func isMediaFile(fileName string) bool {
	// List of common video and audio file extensions
	mediaExtensions := []string{".mp4", ".avi", ".flv", ".mov", ".mkv", ".mp3", ".wav", ".aac", ".flac"}
	for _, ext := range mediaExtensions {
		if strings.HasSuffix(strings.ToLower(fileName), ext) {
			return true
		}
	}
	return false
}

func (api *AlistAPI) GetDownloadUrl(path string, password string) (string, error) {
	newDownloadUrl, err := api.Client.Get(path, password)
	if err != nil {
		return "", err
	}
	return newDownloadUrl.RawURL, nil
}

func (api *AlistAPI) RecursiveCollectContents(remotePath, homePath, srcDir, destPath, currentSubPath, parentFolderName string) ([]FileItem, error) {
	fileItems, err := api.Client.List(remotePath, "", 0, 1000, false)
	if err != nil {
		log.Printf("Directory listing failed: %v", err)
		return nil, err
	}

	var contents []FileItem
	for _, item := range fileItems {
		fullPath := filepath.Join(remotePath, item.Name)
		localDownloadPath := filepath.Join(homePath, currentSubPath, item.Name)
		newDestPath := filepath.Join(destPath, currentSubPath)
		newSrcDir := filepath.Join(srcDir, currentSubPath)

		if !item.IsDir && isMediaFile(item.Name) {
			//newDownloadUrl, err := api.Client.Get(fullPath, "")
			//if err != nil {
			//	break
			//}
			file := FileItem{
				Name:             item.Name,
				FolderName:       filepath.Base(filepath.Dir(fullPath)),
				ParentFolderName: parentFolderName,
				IsDir:            item.IsDir,
				Path:             fullPath,
				DownloadsPath:    localDownloadPath,
				SrcDir:           newSrcDir,
				DstDir:           newDestPath,
				DownloadsUrl:     "",
			}
			contents = append(contents, file)
		} else {
			newSubPath := filepath.Join(currentSubPath, item.Name)
			subItems, err := api.RecursiveCollectContents(fullPath, homePath, srcDir, destPath, newSubPath, item.Name)
			if err != nil {
				//log.Printf("Error collecting contents of directory: %v", err)
				continue
			}
			contents = append(contents, subItems...)
		}
	}

	return contents, nil
}
